/*
 * Copyright (C) 2023-2024
 * Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package logging

import (
	"bytes"
	"github.com/stretchr/testify/assert"
	"log"
	"testing"
)

func setUpLogger(t *testing.T, level string, msg string) *bytes.Buffer {
	logger := GetLogger()

	var buf bytes.Buffer
	logger.BaseLogger.SetOutput(&buf)

	originalFlags := log.Flags()
	t.Cleanup(func() {
		log.SetOutput(nil)
		log.SetFlags(originalFlags)
	})

	if level == LogLevelError {
		logger.Error(msg)
	} else if level == LogLevelWarning {
		logger.Warning(msg)
	} else if level == LogLevelInfo {
		logger.Info(msg)
	} else {
		logger.Debug(msg)
	}

	return &buf
}

func TestError(t *testing.T) {
	buf := setUpLogger(t, LogLevelError, "test error")
	logOutput := buf.String()
	assert.Contains(t, logOutput, "ERROR: test error")
}

func TestWarning(t *testing.T) {
	buf := setUpLogger(t, LogLevelWarning, "test warning")
	logOutput := buf.String()
	assert.Contains(t, logOutput, "WARNING: test warning")
}

func TestInfo(t *testing.T) {
	buf := setUpLogger(t, LogLevelInfo, "test info")
	logOutput := buf.String()
	assert.Contains(t, logOutput, "INFO: test info")
}

func TestDebug(t *testing.T) {
	buf := setUpLogger(t, LogLevelDebug, "test debug")
	logOutput := buf.String()
	assert.Empty(t, logOutput)
}
