package mock

import (
	"context"
	pb "info-manager-proxy/gen/go"
	"fmt"
	"google.golang.org/grpc"
	"google.golang.org/grpc/codes"
	"google.golang.org/grpc/status"
	"net"
)

const GrpcMockPort = 50051

type server struct {
	pb.UnimplementedInformationServiceServer
}

func (s *server) GetInformation(ctx context.Context, in *pb.GrpcInformationRequest) (*pb.GrpcInformationResponse, error) {
		if in.PostfachId == "" {
			return nil, status.Error(codes.InvalidArgument, "PostfachId is missing")
		}
		
		if in.PostfachId == "service_unavailable" {
			return nil, status.Error(codes.Unavailable, "Unavailable")
		} else if in.PostfachId == "internal_server_error" {
			return nil, status.Error(codes.Internal, "Error")
		}
	
		return &pb.GrpcInformationResponse{}, nil
	}

func (s *server) GetInformationById(ctx context.Context, in *pb.GrpcInformationByIdRequest) (*pb.GrpcInformationByIdResponse, error) {
	if in.Id == "" {
		return nil, status.Error(codes.InvalidArgument, "Id is missing")
	}

	return &pb.GrpcInformationByIdResponse{}, nil
}

func StartGrpcServer() *grpc.Server {
	s := grpc.NewServer()
	pb.RegisterInformationServiceServer(s, &server{})

	lis, err := net.Listen("tcp", fmt.Sprintf(":%d", GrpcMockPort))
	if err != nil {
		logger.Fatal("gRPC server failed to listen: %v", err)
	}

	logger.Info("gRPC server listening on port %v", GrpcMockPort)
	if err := s.Serve(lis); err != nil {
		logger.Fatal("gRPC server failed to serve: %v", err)
	}

	return s
}
