package mock

import (
	"context"
	pb "info-manager-proxy/gen/go"
	"fmt"
	"github.com/stretchr/testify/assert"
	"google.golang.org/grpc"
	"google.golang.org/grpc/credentials/insecure"
	"testing"
)

func TestStartGrpcServer(t *testing.T) {
	setUpGrpcEnv := func() (pb.InformationServiceClient, func()) {
		SetUpGrpcServer()

		conn, err := grpc.NewClient(fmt.Sprintf("localhost:%d", GrpcMockPort), grpc.WithTransportCredentials(insecure.NewCredentials()))
		assert.NoError(t, err)

		cleanup := func() {
			conn.Close()
		}
		client := pb.NewInformationServiceClient(conn)

		return client, cleanup
	}

	t.Run("should have no error on get information with valid postfachId", func(t *testing.T) {
		client, cleanUp := setUpGrpcEnv()
		defer cleanUp()

		resp, err := client.GetInformation(context.Background(), &pb.GrpcInformationRequest{PostfachId: "testPostfachId"})

		assert.NoError(t, err)
		assert.NotNil(t, resp)
	})

	t.Run("should have error on get information with empty postfachId", func(t *testing.T) {
		client, cleanUp := setUpGrpcEnv()
		defer cleanUp()

		resp, err := client.GetInformation(context.Background(), &pb.GrpcInformationRequest{PostfachId: ""})

		assert.Error(t, err)
		assert.Nil(t, resp)
	})

	t.Run("should have no error on get information by id", func(t *testing.T) {
		client, cleanUp := setUpGrpcEnv()
		defer cleanUp()

		resp, err := client.GetInformationById(context.Background(), &pb.GrpcInformationByIdRequest{Id: "testId"})

		assert.NoError(t, err)
		assert.NotNil(t, resp)
	})

	t.Run("should have error on get information by id with empty id", func(t *testing.T) {
		client, cleanUp := setUpGrpcEnv()
		defer cleanUp()

		resp, err := client.GetInformationById(context.Background(), &pb.GrpcInformationByIdRequest{Id: ""})

		assert.Error(t, err)
		assert.Nil(t, resp)
	})
}