/*
 * Copyright (C) 2023-2024
 * Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package server

import (
	"context"
	"info-manager-proxy/internal/config"
	"info-manager-proxy/internal/mock"
	"fmt"
	"google.golang.org/grpc/credentials/insecure"
	"net/http"

	"github.com/grpc-ecosystem/grpc-gateway/v2/runtime"
	"google.golang.org/grpc"

	pb "info-manager-proxy/gen/go"
)

func StartHttpGateway(conf config.Config) *http.Server {
	ctx := context.Background()
	ctx, cancel := context.WithCancel(ctx)
	defer cancel()

	grpcEndpoint := conf.Grpc.Url
	if conf.Grpc.Mock {
		grpcEndpoint = fmt.Sprintf("localhost:%d", mock.GrpcMockPort)
	}

	mux := runtime.NewServeMux(runtime.WithErrorHandler(ErrorHandler))
	opts := []grpc.DialOption{grpc.WithTransportCredentials(insecure.NewCredentials())}
	err := pb.RegisterInformationServiceHandlerFromEndpoint(ctx, mux, grpcEndpoint, opts)
	if err != nil {
		logger.Fatal("failed to start HTTP gateway: %v", err)
	}

	RegisterHomeEndpoint(mux)

	httpServer := &http.Server{
		Addr:    fmt.Sprintf(":%d", conf.Server.Port),
		Handler: RequestLoggingMiddleware(mux),
	}

	logger.Info("HTTP gateway listening on port %d", conf.Server.Port)
	if err := httpServer.ListenAndServe(); err != nil {
		logger.Fatal("HTTP gateway failed to serve: %v", err)
	}

	return httpServer
}
